// `timescale 1ns/10ps

module apb3_to_sram(
	input				clk,
	input				rst_n,
	
	inout	wire [7:0]		data,
	output	reg [7:0]	addr,
	output	reg 		cs_n,
	output	reg 		wr_n,
	output	reg 		rd_n,
	input				dreq,
	output	reg 		dack,
	input				interrupt_i,
	
	output				interrupt_o,
	output				cpu_resetn,
	input	[31:0]		apb_paddr,    
	input				apb_penable,  
	output	reg [31:0]	apb_prdata,   
	output	reg 		apb_pready,   
	input				apb_psel,     
	input	[31:0]		apb_pwdata,   
	input				apb_pwrite,   
	output	reg 		apb_pslverror
	
);

localparam	st_idle		= 4'b0001;
localparam	st_setup	= 4'b0010;
localparam	st_access	= 4'b0100;
localparam	st_dreq		= 4'b1000;

reg		[3:0]	state;
reg		[2:0]	cnt;
reg		[3:0]	cnt1;
reg             data_t;
reg     [7:0]   data_o;
wire     [7:0]   data_i;


reg             dreq_r0;
reg             dreq_r1;
reg     [7:0]   dreq_cnt;
reg     [7:0]   cs_n_cnt;
reg				cs_n_r0;
reg				cs_n_r1;
reg				wr_n_r0;
reg				wr_n_r1;
reg				cs_err;

reg		[31:0]	time_cnt = 32'd0;

assign	interrupt_o = interrupt_i;
assign  cpu_resetn = 1'b1;
assign	data_i = data;
assign  data        = apb_pwrite? data_o : 8'bZ;

// always@ (posedge clk or negedge rst_n) begin
	// if(!rst_n) begin
		// cs_n_cnt	<= 0;
		// cs_n_r0		<= 1'b0;
		// cs_n_r1		<= 1'b0;
		// wr_n_r0		<= 1'b0;
		// wr_n_r1		<= 1'b0;
		// cs_err		<= 1'b0;
	// end
	// else begin
		// cs_n_r0		<= cs_n;
		// cs_n_r1		<= cs_n_r0;
		// wr_n_r0		<= wr_n;
		// wr_n_r1		<= wr_n_r0;
		// if(!cs_n) begin
			// if(!wr_n_r0 && wr_n_r1) begin
				// cs_n_cnt	<= cs_n_cnt + 1;
			// end
		// end
		// else if(cs_n) begin
			// if(cs_n_cnt[1:0] != 2'b00) begin
				// cs_err		<= 1'b1;
			// end
		// end
	// end
// end

// always@ (posedge clk) begin
	// if(interrupt_i)
	// begin
		// if(time_cnt < 600000) //600000
		// begin
			// time_cnt		<= time_cnt + 1'b1;
			// cpu_resetn		<= 1'b1;
			// interrupt_o		<= 1'b1;
		// end
		// else if(time_cnt < 600100) //600100
		// begin
			// time_cnt		<= time_cnt + 1'b1;
			// cpu_resetn		<= 1'b0;
			// interrupt_o		<= 1'b0;
		// end
		// else if(time_cnt < 450000000)
		// begin
			// time_cnt		<= time_cnt + 1'b1;
			// cpu_resetn		<= 1'b1;
			// interrupt_o		<= 1'b0;
		// end
		// else
		// begin
			// cpu_resetn		<= 1'b1;
			// time_cnt		<= 32'd0;
			// interrupt_o		<= 1'b0;
		// end
	// end
	// else
	// begin
		// time_cnt		<= 32'd0;
		// cpu_resetn		<= 1'b1;
		// interrupt_o		<= 1'b0;
	// end
// end

always@ (posedge clk or negedge rst_n) begin
	if(!rst_n) begin
		dreq_cnt	<= 0;
		dreq_r0		<= 1'b0;
		dreq_r1		<= 1'b0;
	end
	else begin
		dreq_r0		<= dreq;
		dreq_r1		<= dreq_r0;
		if(dreq_r0 == 1'b1 && dreq_r1 == 1'b0) begin
			dreq_cnt	<= dreq_cnt + 1'b1;
		end
		else begin
			dreq_cnt	<= dreq_cnt;
		end
	end
end

always@ (posedge clk or negedge rst_n) begin
	if(!rst_n) begin
		cnt				<= 3'b0;
		cnt1			<= 4'b0;
		data_o			<= 8'b0;
		addr			<= 8'b0;
		cs_n			<= 1'b1;
		wr_n			<= 1'b1;
		rd_n			<= 1'b1;
		dack			<= 1'b1;
		apb_prdata		<= 32'b0;
		apb_pready		<= 1'b0;
		apb_pslverror	<= 1'b0;
		state			<= st_idle;
	end
	else begin
		if(!dreq) begin
			dack			<= 1'b1;
		end
		else begin
			dack			<= 1'b0;
		end
		// dack			<= 1'b1;
		case(state)
			st_idle : begin
				cnt				<= 3'b0;
				cnt1			<= 4'b0;
				data_o			<= 8'bZ;
				cs_n			<= 1'b1;
				wr_n			<= 1'b1;
				rd_n			<= 1'b1;
				apb_prdata		<= 32'b0;
				apb_pready		<= 1'b0;
				apb_pslverror	<= 1'b0;
				if(apb_psel == 1'b1) begin
					if(apb_paddr[7:0] == 8'h60 || apb_paddr[7:0] == 8'h64) begin
						state			<= st_dreq;
					end
					else begin
						state			<= st_setup;
						cs_n			<= 1'b0;
					end
				end
				else begin
					state			<= st_idle;
				end
			end
			
			st_dreq : begin
				if(apb_penable && apb_psel && cnt1 == 0) begin
					apb_prdata		<= {31'b0, dreq};
					apb_pready		<= 1'b1;
					cnt1			<= cnt1 + 1'b1;
				end
				else if(!apb_psel) begin
					state			<= st_idle;
				end
				else begin
					apb_pready		<= 1'b0;
				end
			end
			
			st_setup : begin
				cnt1			<= cnt1 + 1'b1;
				apb_pready		<= 1'b0;
				apb_prdata		<= 32'b0;
				cnt				<= 3'b0;
				addr			<= apb_paddr[7:0];
				if(cnt1[1:0] == 3) begin
					state			<= st_access;
				end
				if(apb_pwrite) begin
					data_o			<= apb_pwdata[7:0];
					cs_n			<= 1'b0;
					wr_n			<= 1'b0;
					rd_n			<= 1'b1;
				end
				else begin
					data_o			<= 8'bZ;
					cs_n			<= 1'b0;
					wr_n			<= 1'b1;
					rd_n			<= 1'b0;
				end
			end
			
			st_access : begin
				cnt1			<= cnt1 + 1'b1;
				if(cnt1[1:0] == 3) begin
					cnt				<= cnt + 1'b1;
				end
				else begin
					cnt				<= cnt;
				end
				if(cnt == 0) begin
					if(cnt1[1:0] == 1)begin
						wr_n			<= 1'b1;
						rd_n			<= 1'b1;
					end
					if(cnt1[1:0] == 1)begin
						if(apb_pwrite) begin
							apb_pready		<= 1'b0;
						end
						else begin
							// apb_prdata		<= {data_i, apb_prdata[31:24]};
							apb_prdata[7:0]		<= data_i;
							apb_pready		<= 1'b0;
							data_o			<= 8'bZ;
						end
					end
				end
				else if(cnt == 1) begin
					if(cnt1[1:0] == 0)begin
						addr			<= addr + 1'b1;
					end
					if(apb_pwrite) begin
						data_o			<= apb_pwdata[15:8];
						cs_n			<= 1'b0;
						wr_n			<= 1'b0;
						rd_n			<= 1'b1;
					end
					else begin
						cs_n			<= 1'b0;
						wr_n			<= 1'b1;
						rd_n			<= 1'b0;
					end
				end
				else if(cnt == 2) begin
					if(cnt1[1:0] == 1)begin
						wr_n			<= 1'b1;
						rd_n			<= 1'b1;
					end
					if(cnt1[1:0] == 1)begin
						if(apb_pwrite) begin
							apb_pready		<= 1'b0;
						end
						else begin
							// apb_prdata		<= {data_i, apb_prdata[31:24]};
							apb_prdata[15:8]		<= data_i;
							apb_pready		<= 1'b0;
							data_o			<= 8'bZ;
						end
					end
				end
				else if(cnt == 3) begin
					if(cnt1[1:0] == 0)begin
						addr			<= addr + 1'b1;
					end
					if(apb_pwrite) begin
						data_o			<= apb_pwdata[23:16];
						cs_n			<= 1'b0;
						wr_n			<= 1'b0;
						rd_n			<= 1'b1;
					end
					else begin
						cs_n			<= 1'b0;
						wr_n			<= 1'b1;
						rd_n			<= 1'b0;
					end
				end
				else if(cnt == 4) begin
					if(cnt1[1:0] == 1)begin
						wr_n			<= 1'b1;
						rd_n			<= 1'b1;
					end
					if(cnt1[1:0] == 1)begin
						if(apb_pwrite) begin
							apb_pready		<= 1'b0;
						end
						else begin
							// apb_prdata		<= {data_i, apb_prdata[31:24]};
							apb_prdata[23:16]		<= data_i;
							apb_pready		<= 1'b0;
							data_o			<= 8'bZ;
						end
					end
				end
				else if(cnt == 5) begin
					if(cnt1[1:0] == 0)begin
						addr			<= addr + 1'b1;
					end
					if(apb_pwrite) begin
						data_o			<= apb_pwdata[31:24];
						cs_n			<= 1'b0;
						wr_n			<= 1'b0;
						rd_n			<= 1'b1;
					end
					else begin
						cs_n			<= 1'b0;
						wr_n			<= 1'b1;
						rd_n			<= 1'b0;
					end
				end
				else if(cnt == 6) begin
					if(cnt1[1:0] == 1)begin
						wr_n			<= 1'b1;
						rd_n			<= 1'b1;
					end
					if(cnt1[1:0] == 1)begin
						// apb_prdata		<= {data_i, apb_prdata[31:24]};
						apb_prdata[31:24]		<= data_i;
					end
					else if(cnt1[1:0] == 3)begin
						if(apb_pwrite) begin
							apb_pready		<= 1'b1;
						end
						else begin
							apb_pready		<= 1'b1;
							data_o			<= 8'bZ;
						end
					end
				end
				else if(cnt == 7) begin
					data_o			<= 8'bZ;
					wr_n			<= 1'b1;
					rd_n			<= 1'b1;
					apb_pready		<= 1'b0;
					state			<= st_idle;
				end
			end
			
			default : begin state <= st_idle; end
		endcase
	end
end


// IOBUF data0(
	// .O		(data_i[0]),
	// .IO		(data[0]),
	// .I		(data_o[0]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data1(
	// .O		(data_i[1]),
	// .IO		(data[1]),
	// .I		(data_o[1]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data2(
	// .O		(data_i[2]),
	// .IO		(data[2]),
	// .I		(data_o[2]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data3(
	// .O		(data_i[3]),
	// .IO		(data[3]),
	// .I		(data_o[3]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data4(
	// .O		(data_i[4]),
	// .IO		(data[4]),
	// .I		(data_o[4]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data5(
	// .O		(data_i[5]),
	// .IO		(data[5]),
	// .I		(data_o[5]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data6(
	// .O		(data_i[6]),
	// .IO		(data[6]),
	// .I		(data_o[6]),
	// .OEN	(~apb_pwrite)
// );
// IOBUF data7(
	// .O		(data_i[7]),
	// .IO		(data[7]),
	// .I		(data_o[7]),
	// .OEN	(~apb_pwrite)
// );


endmodule